"""Custom fields snapshot."""

DATA_CUSTOM_FIELDS = {
    "count": 8,
    "next": None,
    "previous": None,
    "all": [8, 7, 6, 5, 4, 3, 2, 1],
    "results": [
        {"id": 8, "name": "Custom Link", "data_type": "documentlink"},
        {"id": 7, "name": "Custom URL", "data_type": "url"},
        {"id": 6, "name": "Custom Text -added-", "data_type": "string"},
        {"id": 5, "name": "Custom MONEYY $$$", "data_type": "monetary"},
        {"id": 4, "name": "Custom Floating", "data_type": "float"},
        {"id": 3, "name": "Custom Int", "data_type": "integer"},
        {"id": 2, "name": "Custom Date", "data_type": "date"},
        {"id": 1, "name": "Custom Bool", "data_type": "boolean"},
    ],
}
