"""Object permissions snapshot."""

DATA_OBJECT_PERMISSIONS = {
    "view": {
        "users": [1, 2],
        "groups": [],
    },
    "change": {
        "users": [],
        "groups": [1],
    },
}
