"""Tags snapshot."""

DATA_TAGS = {
    "count": 2,
    "next": None,
    "previous": None,
    "all": [
        1,
        2,
        3,
        4,
        5,
    ],
    "results": [
        {
            "id": 1,
            "slug": "important",
            "name": "Important",
            "color": "#ff0000",
            "text_color": "#00ff00",
            "match": "",
            "matching_algorithm": 0,
            "is_insensitive": True,
            "is_inbox_tag": False,
            "document_count": 20,
            "owner": None,
            "user_can_change": True,
        },
        {
            "id": 2,
            "slug": "inbox",
            "name": "Inbox",
            "color": "#ff0000",
            "text_color": "#00ff00",
            "match": "",
            "matching_algorithm": 1,
            "is_insensitive": True,
            "is_inbox_tag": True,
            "document_count": 20,
            "owner": None,
            "user_can_change": True,
        },
        {
            "id": 3,
            "slug": "test-3",
            "name": "Test 3",
            "color": "#ff0000",
            "text_color": "#00ff00",
            "match": "",
            "matching_algorithm": 2,
            "is_insensitive": True,
            "is_inbox_tag": False,
            "document_count": 20,
            "owner": None,
            "user_can_change": True,
        },
        {
            "id": 4,
            "slug": "test-4",
            "name": "Test 4",
            "color": "#ff0000",
            "text_color": "#00ff00",
            "match": "",
            "matching_algorithm": 3,
            "is_insensitive": True,
            "is_inbox_tag": False,
            "document_count": 20,
            "owner": None,
            "user_can_change": True,
        },
        {
            "id": 5,
            "slug": "test-5",
            "name": "Test 5",
            "color": "#ff0000",
            "text_color": "#00ff00",
            "match": "",
            "matching_algorithm": 4,
            "is_insensitive": True,
            "is_inbox_tag": False,
            "document_count": 20,
            "owner": None,
            "user_can_change": True,
        },
    ],
}
