"""Users snapshot."""

DATA_USERS = {
    "count": 4,
    "next": None,
    "previous": None,
    "all": [1, 2, 3, 4],
    "results": [
        {
            "id": 1,
            "username": "admin",
            "email": "you@inside.me",
            "password": "**********",
            "first_name": "",
            "last_name": "",
            "date_joined": "2022-02-11T22:28:25+00:00",
            "is_staff": True,
            "is_active": False,
            "is_superuser": False,
            "groups": [],
            "user_permissions": [],
            "inherited_permissions": [],
        },
        {
            "id": 2,
            "username": "alpha",
            "email": "alpha@is-not.beta",
            "password": "**********",
            "first_name": "Alpha",
            "last_name": "Centauri",
            "date_joined": "2023-06-27T15:59:01.975496+00:00",
            "is_staff": False,
            "is_active": True,
            "is_superuser": False,
            "groups": [1],
            "user_permissions": [],
            "inherited_permissions": [
                "documents.delete_customfieldinstance",
                "documents.delete_consumptiontemplate",
                "documents.delete_customfield",
                "documents.add_uisettings",
                "documents.view_uisettings",
                "documents.view_tag",
                "documents.change_customfield",
                "documents.view_sharelink",
                "documents.change_note",
                "documents.view_correspondent",
                "documents.change_customfieldinstance",
                "documents.delete_note",
                "documents.add_consumptiontemplate",
                "documents.add_customfield",
                "documents.change_sharelink",
                "documents.view_consumptiontemplate",
                "documents.view_documenttype",
                "documents.change_document",
                "documents.view_customfieldinstance",
                "documents.add_sharelink",
                "documents.view_document",
                "documents.delete_uisettings",
                "documents.add_savedview",
                "documents.view_note",
                "documents.view_customfield",
                "documents.add_customfieldinstance",
                "documents.change_uisettings",
                "documents.change_savedview",
                "documents.add_document",
                "documents.add_note",
                "documents.change_consumptiontemplate",
                "documents.delete_sharelink",
                "documents.delete_savedview",
                "documents.view_savedview",
                "documents.delete_document",
            ],
        },
        {
            "id": 3,
            "username": "omega",
            "email": "omega.weapon@ff8.net",
            "password": "**********",
            "first_name": "Omega",
            "last_name": "Weapon",
            "date_joined": "2022-02-14T13:42:54+00:00",
            "is_staff": True,
            "is_active": True,
            "is_superuser": True,
            "groups": [1],
            "user_permissions": [],
            "inherited_permissions": [
                "documents.change_paperlesstask",
                "documents.delete_customfieldinstance",
                "django_celery_results.delete_chordcounter",
                "paperless_mail.change_processedmail",
                "django_celery_results.view_chordcounter",
                "documents.view_storagepath",
                "documents.delete_tag",
                "sessions.delete_session",
                "documents.view_log",
                "django_celery_results.change_groupresult",
                "documents.add_uisettings",
                "authtoken.delete_token",
                "admin.add_logentry",
                "django_celery_results.add_groupresult",
                "guardian.view_userobjectpermission",
                "paperless_mail.add_processedmail",
                "documents.change_customfieldinstance",
                "documents.view_correspondent",
                "auth.add_permission",
                "admin.view_logentry",
                "documents.delete_note",
                "documents.add_customfield",
                "auth.view_permission",
                "documents.add_consumptiontemplate",
                "guardian.delete_groupobjectpermission",
                "documents.change_sharelink",
                "documents.view_consumptiontemplate",
                "documents.delete_log",
                "guardian.delete_userobjectpermission",
                "paperless_mail.delete_mailrule",
                "sessions.change_session",
                "documents.add_tag",
                "documents.view_documenttype",
                "auth.view_user",
                "auth.add_group",
                "guardian.view_groupobjectpermission",
                "documents.change_document",
                "documents.change_savedviewfilterrule",
                "paperless_mail.add_mailrule",
                "documents.view_customfieldinstance",
                "guardian.add_userobjectpermission",
                "documents.add_sharelink",
                "auth.change_user",
                "documents.view_document",
                "django_celery_results.delete_taskresult",
                "documents.delete_uisettings",
                "documents.add_savedview",
                "documents.view_note",
                "authtoken.view_token",
                "authtoken.change_tokenproxy",
                "documents.change_storagepath",
                "documents.change_uisettings",
                "django_celery_results.add_chordcounter",
                "paperless_mail.change_mailrule",
                "documents.change_documenttype",
                "documents.change_savedview",
                "auth.delete_permission",
                "documents.add_storagepath",
                "django_celery_results.change_chordcounter",
                "documents.add_note",
                "documents.change_consumptiontemplate",
                "documents.delete_savedview",
                "guardian.change_userobjectpermission",
                "admin.delete_logentry",
                "authtoken.add_token",
                "documents.view_savedview",
                "documents.delete_paperlesstask",
                "sessions.view_session",
                "auth.delete_group",
                "paperless_mail.delete_mailaccount",
                "documents.add_savedviewfilterrule",
                "authtoken.delete_tokenproxy",
                "documents.delete_consumptiontemplate",
                "authtoken.view_tokenproxy",
                "authtoken.change_token",
                "documents.delete_customfield",
                "contenttypes.change_contenttype",
                "documents.change_log",
                "paperless_mail.view_mailrule",
                "documents.view_uisettings",
                "documents.view_tag",
                "documents.change_customfield",
                "admin.change_logentry",
                "documents.view_sharelink",
                "documents.change_note",
                "documents.delete_savedviewfilterrule",
                "auth.add_user",
                "django_celery_results.delete_groupresult",
                "documents.change_correspondent",
                "documents.add_documenttype",
                "django_celery_results.view_groupresult",
                "documents.add_paperlesstask",
                "contenttypes.add_contenttype",
                "auth.view_group",
                "guardian.change_groupobjectpermission",
                "django_celery_results.change_taskresult",
                "documents.change_tag",
                "paperless_mail.delete_processedmail",
                "paperless_mail.change_mailaccount",
                "documents.delete_correspondent",
                "paperless_mail.view_processedmail",
                "documents.delete_storagepath",
                "sessions.add_session",
                "paperless_mail.add_mailaccount",
                "django_celery_results.view_taskresult",
                "documents.view_customfield",
                "documents.view_paperlesstask",
                "documents.add_customfieldinstance",
                "django_celery_results.add_taskresult",
                "documents.view_savedviewfilterrule",
                "documents.add_log",
                "guardian.add_groupobjectpermission",
                "auth.delete_user",
                "auth.change_permission",
                "documents.add_document",
                "documents.delete_documenttype",
                "documents.delete_sharelink",
                "documents.add_correspondent",
                "authtoken.add_tokenproxy",
                "auth.change_group",
                "contenttypes.delete_contenttype",
                "contenttypes.view_contenttype",
                "paperless_mail.view_mailaccount",
                "documents.delete_document",
            ],
        },
        {
            "id": 4,
            "username": "zg_test",
            "email": "",
            "password": "**********",
            "first_name": "",
            "last_name": "",
            "date_joined": "2023-06-27T15:56:39.754859+00:00",
            "is_staff": False,
            "is_active": True,
            "is_superuser": False,
            "groups": [1],
            "user_permissions": [],
            "inherited_permissions": [
                "documents.delete_customfieldinstance",
                "documents.delete_consumptiontemplate",
                "documents.delete_customfield",
                "documents.add_uisettings",
                "documents.view_uisettings",
                "documents.view_tag",
                "documents.change_customfield",
                "documents.view_sharelink",
                "documents.change_note",
                "documents.view_correspondent",
                "documents.change_customfieldinstance",
                "documents.delete_note",
                "documents.add_consumptiontemplate",
                "documents.add_customfield",
                "documents.change_sharelink",
                "documents.view_consumptiontemplate",
                "documents.view_documenttype",
                "documents.change_document",
                "documents.view_customfieldinstance",
                "documents.add_sharelink",
                "documents.view_document",
                "documents.delete_uisettings",
                "documents.add_savedview",
                "documents.view_note",
                "documents.view_customfield",
                "documents.add_customfieldinstance",
                "documents.change_uisettings",
                "documents.change_savedview",
                "documents.add_document",
                "documents.add_note",
                "documents.change_consumptiontemplate",
                "documents.delete_sharelink",
                "documents.delete_savedview",
                "documents.view_savedview",
                "documents.delete_document",
            ],
        },
    ],
}
