"""Document types snapshot."""

DATA_DOCUMENT_TYPES = {
    "count": 5,
    "next": None,
    "previous": None,
    "all": [1, 2, 3, 4, 5],
    "results": [
        {
            "id": 1,
            "slug": "bescheid",
            "name": "Bescheid",
            "match": "",
            "matching_algorithm": 1,
            "is_insensitive": True,
            "document_count": 42,
            "owner": None,
            "user_can_change": True,
        },
        {
            "id": 2,
            "slug": "document",
            "name": "Document",
            "match": "",
            "matching_algorithm": 1,
            "is_insensitive": True,
            "document_count": 218,
            "owner": None,
            "user_can_change": None,
        },
        {
            "id": 3,
            "slug": "kontoauszug",
            "name": "Kontoauszug",
            "match": "",
            "matching_algorithm": 6,
            "is_insensitive": True,
            "document_count": 227,
            "owner": None,
            "user_can_change": None,
        },
        {
            "id": 4,
            "slug": "kundigung",
            "name": "Kuendigung",
            "match": "",
            "matching_algorithm": 1,
            "is_insensitive": True,
            "document_count": 24,
            "owner": None,
            "user_can_change": True,
        },
        {
            "id": 5,
            "slug": "rechnung",
            "name": "Rechnung",
            "match": "",
            "matching_algorithm": 6,
            "is_insensitive": True,
            "document_count": 246,
            "owner": None,
            "user_can_change": True,
        },
    ],
}
