"""Document metadata snapshot."""

DATA_DOCUMENT_METADATA = {
    "original_checksum": "18e2352cc13379d19bd9ce329428bb99",
    "original_size": 190348,
    "original_mime_type": "application/pdf",
    "media_filename": "xxx.pdf",
    "has_archive_version": True,
    "original_metadata": [],
    "archive_checksum": "8a69542583571337dd263f8bb3cf23bd",
    "archive_media_filename": "0000226.pdf",
    "original_filename": None,
    "lang": "de",
    "archive_size": 161371,
    "archive_metadata": [
        {
            "namespace": "http://ns.adobe.com/pdf/1.3/",
            "prefix": "pdf",
            "key": "Producer",
            "value": "pikepdf 2.16.1",
        },
        {
            "namespace": "http://ns.adobe.com/xap/1.0/",
            "prefix": "xmp",
            "key": "ModifyDate",
            "value": "2022-02-12T11:34:40+00:00",
        },
        {
            "namespace": "http://ns.adobe.com/xap/1.0/",
            "prefix": "xmp",
            "key": "CreateDate",
            "value": "2022-01-07T13:40:49",
        },
        {
            "namespace": "http://ns.adobe.com/xap/1.0/",
            "prefix": "xmp",
            "key": "CreatorTool",
            "value": "ocrmypdf 12.3.2 / Tesseract OCR-PDF 4.1.1",
        },
        {
            "namespace": "http://ns.adobe.com/xap/1.0/mm/",
            "prefix": "xmpMM",
            "key": "DocumentID",
            "value": "uuid:13371337-2342-1337-4242-133723426665",
        },
        {
            "namespace": "http://purl.org/dc/elements/1.1/",
            "prefix": "dc",
            "key": "format",
            "value": "application/pdf",
        },
        {
            "namespace": "http://purl.org/dc/elements/1.1/",
            "prefix": "dc",
            "key": "title",
            "value": "Octopus Energy updated",
        },
        {
            "namespace": "http://www.aiim.org/pdfa/ns/id/",
            "prefix": "pdfaid",
            "key": "part",
            "value": "2",
        },
        {
            "namespace": "http://www.aiim.org/pdfa/ns/id/",
            "prefix": "pdfaid",
            "key": "conformance",
            "value": "B",
        },
        {
            "namespace": "http://purl.org/dc/elements/1.1/",
            "prefix": "dc",
            "key": "creator",
            "value": "None",
        },
        {
            "namespace": "http://ns.adobe.com/xap/1.0/",
            "prefix": "xmp",
            "key": "MetadataDate",
            "value": "2022-02-12T11:34:40.383718+00:00",
        },
    ],
}
