"""Config snapshot."""

DATA_CONFIG = [
    {
        "id": 1,
        "user_args": None,
        "output_type": "pdf",
        "pages": None,
        "language": "eng",
        "mode": None,
        "skip_archive_file": None,
        "image_dpi": None,
        "unpaper_clean": None,
        "deskew": None,
        "rotate_pages": None,
        "rotate_pages_threshold": None,
        "max_image_pixels": None,
        "color_conversion_strategy": None,
        "app_title": None,
        "app_logo": None,
    }
]
