"""Correspondents snapshot."""

DATA_CORRESPONDENTS = {
    "count": 5,
    "next": None,
    "previous": None,
    "all": [1, 2, 3, 4, 5],
    "results": [
        {
            "id": 1,
            "slug": "sample-correspondent",
            "name": "Sample Correspondent",
            "match": "",
            "matching_algorithm": 1,
            "is_insensitive": True,
            "document_count": 12,
            "last_correspondence": "2022-10-18",
            "owner": None,
            "user_can_change": True,
        },
        {
            "id": 2,
            "slug": "burger-fastfood-delivery-billing",
            "name": "Burger FastFood Delivery Billing",
            "match": "",
            "matching_algorithm": 1,
            "is_insensitive": True,
            "document_count": 6,
            "last_correspondence": "2006-03-07",
            "owner": None,
            "user_can_change": True,
        },
        {
            "id": 3,
            "slug": "example-company",
            "name": "Example Company",
            "match": "",
            "matching_algorithm": 6,
            "is_insensitive": True,
            "document_count": 3,
            "last_correspondence": "2022-03-11",
            "owner": 3,
            "user_can_change": True,
        },
        {
            "id": 4,
            "slug": "no-illness-more-money-health-insurance",
            "name": "No Illness More Money Health Insurance",
            "match": "",
            "matching_algorithm": 1,
            "is_insensitive": True,
            "document_count": 6,
            "last_correspondence": "2000-07-18",
            "owner": None,
            "user_can_change": True,
        },
        {
            "id": 5,
            "slug": "your-cash-is-my-cash-bank",
            "name": "Your Cash is my Cash Bank",
            "match": "Your Cash My Bank",
            "matching_algorithm": 2,
            "is_insensitive": True,
            "document_count": 20,
            "last_correspondence": "2023-08-28",
            "owner": None,
            "user_can_change": True,
        },
    ],
}
