"""Groups snapshot."""

DATA_GROUPS = {
    "count": 1,
    "next": None,
    "previous": None,
    "all": [1],
    "results": [
        {
            "id": 1,
            "name": "Sample Group",
            "permissions": [
                "add_consumptiontemplate",
                "change_consumptiontemplate",
                "delete_consumptiontemplate",
                "view_consumptiontemplate",
                "view_correspondent",
                "add_customfield",
                "change_customfield",
                "delete_customfield",
                "view_customfield",
                "add_customfieldinstance",
                "change_customfieldinstance",
                "delete_customfieldinstance",
                "view_customfieldinstance",
                "add_document",
                "change_document",
                "delete_document",
                "view_document",
                "view_documenttype",
                "add_note",
                "change_note",
                "delete_note",
                "view_note",
                "add_savedview",
                "change_savedview",
                "delete_savedview",
                "view_savedview",
                "add_sharelink",
                "change_sharelink",
                "delete_sharelink",
                "view_sharelink",
                "view_tag",
                "add_uisettings",
                "change_uisettings",
                "delete_uisettings",
                "view_uisettings",
            ],
        },
    ],
}
